/* This module implements and interface to the SLang KBI routines */

/* 2005/03/15 : module creation (Laurent Perez)
 * 2005/03/23 : change from obsolete SLang_doerror to SLang_verror (patch from Paul Boekholt)
 * 		rename intrinsics and constants to be consistant with slsmg module (slang-2)
 * 		new constants SLKBI_KEY_*
 * 		remove init_slkbi_module (init_<module>_module no longer supported in slang-2)
 */

#include <stdio.h>
#include <slang.h>
   
SLANG_MODULE(slkbi);

#define MODULE_MAJOR_VERSION    0
#define MODULE_MINOR_VERSION    2
#define MODULE_PATCH_LEVEL      0
static char *Module_Version_String = "0.2.0";
#define MODULE_VERSION_NUMBER   \
      (MODULE_MAJOR_VERSION*10000+MODULE_MINOR_VERSION*100+MODULE_PATCH_LEVEL)

static unsigned int kbi_getkey (void)
{
   return SLang_getkey();
}

static int kbi_kp_getkey (void)
{
   return SLkp_getkey();
}

static int kbi_kp_init (void)
{
   SLtt_get_terminfo ();
   if (-1 == SLkp_init ())
     {	
	SLang_verror (SL_INTRINSIC_ERROR, "SLkp_init failed.");
	exit (1);
     }
   return 0;
}

static void kbi_reset_tty (void)
{
   SLang_reset_tty();
}

static SLang_Intrin_Fun_Type Kbi_Intrinsics [] =
{
   MAKE_INTRINSIC_0("slkbi_getkey", kbi_getkey, SLANG_UINT_TYPE),
   MAKE_INTRINSIC_0("slkbi_kp_getkey", kbi_kp_getkey, SLANG_INT_TYPE),
   MAKE_INTRINSIC_0("slkbi_kp_init", kbi_kp_init, SLANG_INT_TYPE),
   MAKE_INTRINSIC_0("slkbi_reset_tty", kbi_reset_tty, SLANG_VOID_TYPE),
   SLANG_END_INTRIN_FUN_TABLE
};

static SLang_IConstant_Type Kbi_Constants [] =
{
   MAKE_ICONSTANT("SLKBI_KEY_ERR", SL_KEY_ERR),
   MAKE_ICONSTANT("SLKBI_KEY_UP", SL_KEY_UP),
   MAKE_ICONSTANT("SLKBI_KEY_DOWN", SL_KEY_DOWN),
   MAKE_ICONSTANT("SLKBI_KEY_LEFT", SL_KEY_LEFT),
   MAKE_ICONSTANT("SLKBI_KEY_RIGHT", SL_KEY_RIGHT),
   MAKE_ICONSTANT("SLKBI_KEY_PPAGE", SL_KEY_PPAGE),
   MAKE_ICONSTANT("SLKBI_KEY_NPAGE", SL_KEY_NPAGE),
   MAKE_ICONSTANT("SLKBI_KEY_HOME", SL_KEY_HOME),
   MAKE_ICONSTANT("SLKBI_KEY_END", SL_KEY_END),
   MAKE_ICONSTANT("SLKBI_KEY_BACKSPACE", 127),
   MAKE_ICONSTANT("SLKBI_KEY_ENTER", 13),
   MAKE_ICONSTANT("SLKBI_KEY_IC", SL_KEY_IC),
   MAKE_ICONSTANT("SLKBI_KEY_DELETE", SL_KEY_DELETE),
   MAKE_ICONSTANT("SLKBI_KEY_TAB", 9),
   MAKE_ICONSTANT("SLKBI_KEY_SPACE", 32),
   MAKE_ICONSTANT("SLKBI_KEY_F1", SL_KEY_F0 + 1),
   MAKE_ICONSTANT("SLKBI_KEY_F2", SL_KEY_F0 + 2),
   MAKE_ICONSTANT("SLKBI_KEY_F3", SL_KEY_F0 + 3),
   MAKE_ICONSTANT("SLKBI_KEY_F4", SL_KEY_F0 + 4),
   MAKE_ICONSTANT("SLKBI_KEY_F5", SL_KEY_F0 + 5),
   MAKE_ICONSTANT("SLKBI_KEY_F6", SL_KEY_F0 + 6),
   MAKE_ICONSTANT("SLKBI_KEY_F7", SL_KEY_F0 + 7),
   MAKE_ICONSTANT("SLKBI_KEY_F8", SL_KEY_F0 + 8),
   MAKE_ICONSTANT("SLKBI_KEY_F9", SL_KEY_F0 + 9),
   MAKE_ICONSTANT("SLKBI_KEY_F10", SL_KEY_F0 + 10),
   MAKE_ICONSTANT("SLKBI_KEY_F11", SL_KEY_F0 + 11),
   MAKE_ICONSTANT("SLKBI_KEY_F12", SL_KEY_F0 + 12),
   SLANG_END_ICONST_TABLE
};

int init_slkbi_module_ns (char *ns_name)
{  
   int abort_char = 7;
   
   SLang_NameSpace_Type *ns = SLns_create_namespace (ns_name);
   if (ns == NULL)
     return -1;

   if ((-1 == SLns_add_intrin_fun_table (ns, Kbi_Intrinsics, "__SLKBI__"))
     || (-1 == SLns_add_iconstant_table (ns, Kbi_Constants, NULL)))
     return -1;
   
   if (-1 == SLang_init_tty (abort_char, 0, 1))
     {
   	fprintf (stderr, "Unable to initialize the terminal.\n");
   	exit (-1);
     }
   SLang_set_abort_signal (NULL);
   
   return 0;
}

void deinit_kbi_module (void)
{
   SLang_reset_tty();
}
